static VALUE
set_url(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@url", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_URL, dst);
    return Qnil;
}

static VALUE
set_port(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@port", v);
    curl_easy_setopt(s->curl, CURLOPT_PORT, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_proxy(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@proxy", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_PROXY, dst);
    return Qnil;
}

static VALUE
set_userpwd(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@userpwd", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_USERPWD, dst);
    return Qnil;
}

static VALUE
set_proxyuserpwd(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@proxyuserpwd", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_PROXYUSERPWD, dst);
    return Qnil;
}

static VALUE
set_range(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@range", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_RANGE, dst);
    return Qnil;
}

static VALUE
set_timeout(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@timeout", v);
    curl_easy_setopt(s->curl, CURLOPT_TIMEOUT, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_postfields(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@postfields", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len);   /* able to free */
      memcpy(dst, src, len);
      curl_easy_setopt(s->curl, CURLOPT_POSTFIELDSIZE, len);
    } else {
      dst = NULL;
    }
    curl_easy_setopt(s->curl, CURLOPT_POSTFIELDS, dst);

    return Qnil;
}

static VALUE
set_referer(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@referer", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_REFERER, dst);
    return Qnil;
}

static VALUE
set_ftpport(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@ftpport", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_FTPPORT, dst);
    return Qnil;
}

static VALUE
set_useragent(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@useragent", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_USERAGENT, dst);
    return Qnil;
}

static VALUE
set_low_speed_limit(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@low_speed_limit", v);
    curl_easy_setopt(s->curl, CURLOPT_LOW_SPEED_LIMIT, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_low_speed_time(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@low_speed_time", v);
    curl_easy_setopt(s->curl, CURLOPT_LOW_SPEED_TIME, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_resume_from(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@resume_from", v);
    curl_easy_setopt(s->curl, CURLOPT_RESUME_FROM, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_cookie(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@cookie", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_COOKIE, dst);
    return Qnil;
}

static VALUE
set_httpheader(VALUE self, VALUE v)
{
    rbCurl *s;
    int i;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@httpheader", v);

    if (s->list_httpheader) {
        curl_slist_free_all(s->list_httpheader);
	s->list_httpheader = NULL;
    }
    if (v != Qnil) {
        for (i = 0; i < RARRAY(v)->len; i++) {
            s->list_httpheader = curl_slist_append(s->list_httpheader, rb_str2cstr(RARRAY(v)->ptr[i], 0));
        }
    }

    curl_easy_setopt(s->curl, CURLOPT_HTTPHEADER, s->list_httpheader);
    return Qnil;
}

static VALUE
set_sslcert(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@sslcert", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_SSLCERT, dst);
    return Qnil;
}

static VALUE
set_sslcertpasswd(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@sslcertpasswd", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_SSLCERTPASSWD, dst);
    return Qnil;
}

static VALUE
set_crlf(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@crlf", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_CRLF, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_CRLF, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_quote(VALUE self, VALUE v)
{
    rbCurl *s;
    int i;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@quote", v);

    if (s->list_quote) {
        curl_slist_free_all(s->list_quote);
	s->list_quote = NULL;
    }
    if (v != Qnil) {
        for (i = 0; i < RARRAY(v)->len; i++) {
            s->list_quote = curl_slist_append(s->list_quote, rb_str2cstr(RARRAY(v)->ptr[i], 0));
        }
    }

    curl_easy_setopt(s->curl, CURLOPT_QUOTE, s->list_quote);
    return Qnil;
}

static VALUE
set_cookiefile(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@cookiefile", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_COOKIEFILE, dst);
    return Qnil;
}

static VALUE
set_sslversion(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@sslversion", v);
    curl_easy_setopt(s->curl, CURLOPT_SSLVERSION, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_timecondition(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@timecondition", v);
    curl_easy_setopt(s->curl, CURLOPT_TIMECONDITION, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_timevalue(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@timevalue", v);
    curl_easy_setopt(s->curl, CURLOPT_TIMEVALUE, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_customrequest(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@customrequest", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_CUSTOMREQUEST, dst);
    return Qnil;
}

static VALUE
set_postquote(VALUE self, VALUE v)
{
    rbCurl *s;
    int i;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@postquote", v);

    if (s->list_postquote) {
        curl_slist_free_all(s->list_postquote);
	s->list_postquote = NULL;
    }
    if (v != Qnil) {
        for (i = 0; i < RARRAY(v)->len; i++) {
            s->list_postquote = curl_slist_append(s->list_postquote, rb_str2cstr(RARRAY(v)->ptr[i], 0));
        }
    }

    curl_easy_setopt(s->curl, CURLOPT_POSTQUOTE, s->list_postquote);
    return Qnil;
}

static VALUE
set_verbose(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@verbose", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_VERBOSE, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_VERBOSE, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_header(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@header", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_HEADER, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_HEADER, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_nobody(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@nobody", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_NOBODY, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_NOBODY, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_failonerror(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@failonerror", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_FAILONERROR, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_FAILONERROR, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_upload(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@upload", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_UPLOAD, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_UPLOAD, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_ftplistonly(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@ftplistonly", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_FTPLISTONLY, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_FTPLISTONLY, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_ftpappend(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@ftpappend", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_FTPAPPEND, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_FTPAPPEND, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_netrc(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@netrc", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_NETRC, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_NETRC, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_followlocation(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@followlocation", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_FOLLOWLOCATION, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_FOLLOWLOCATION, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_transfertext(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@transfertext", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_TRANSFERTEXT, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_TRANSFERTEXT, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_put(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@put", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_PUT, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_PUT, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_proxyport(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@proxyport", v);
    curl_easy_setopt(s->curl, CURLOPT_PROXYPORT, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_httpproxytunnel(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@httpproxytunnel", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_HTTPPROXYTUNNEL, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_HTTPPROXYTUNNEL, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_interface(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@interface", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_INTERFACE, dst);
    return Qnil;
}

static VALUE
set_krb4level(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@krb4level", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_KRB4LEVEL, dst);
    return Qnil;
}

static VALUE
set_ssl_verifypeer(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@ssl_verifypeer", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_SSL_VERIFYPEER, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_SSL_VERIFYPEER, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_cainfo(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@cainfo", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_CAINFO, dst);
    return Qnil;
}

static VALUE
set_maxredirs(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@maxredirs", v);
    curl_easy_setopt(s->curl, CURLOPT_MAXREDIRS, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_filetime(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@filetime", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_FILETIME, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_FILETIME, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_maxconnects(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@maxconnects", v);
    curl_easy_setopt(s->curl, CURLOPT_MAXCONNECTS, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_closepolicy(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@closepolicy", v);
    curl_easy_setopt(s->curl, CURLOPT_CLOSEPOLICY, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_fresh_connect(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@fresh_connect", v);
    curl_easy_setopt(s->curl, CURLOPT_FRESH_CONNECT, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_forbid_reuse(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@forbid_reuse", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_FORBID_REUSE, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_FORBID_REUSE, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_random_file(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@random_file", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_RANDOM_FILE, dst);
    return Qnil;
}

static VALUE
set_egdsocket(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@egdsocket", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_EGDSOCKET, dst);
    return Qnil;
}

static VALUE
set_connecttimeout(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@connecttimeout", v);
    curl_easy_setopt(s->curl, CURLOPT_CONNECTTIMEOUT, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_httpget(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@httpget", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_HTTPGET, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_HTTPGET, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

static VALUE
set_ssl_verifyhost(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@ssl_verifyhost", v);
    curl_easy_setopt(s->curl, CURLOPT_SSL_VERIFYHOST, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_cookiejar(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@cookiejar", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_COOKIEJAR, dst);
    return Qnil;
}

static VALUE
set_ssl_cipher_list(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@ssl_cipher_list", v);

    if (v != Qnil) {
      src = rb_str2cstr(v, &len);   /* copy string */
      dst = ALLOC_N(char, len+1);   /* able to free */
      memcpy(dst, src, len);
      dst[len] = '\0';
    } else {
      dst = NULL;
    }

    curl_easy_setopt(s->curl, CURLOPT_SSL_CIPHER_LIST, dst);
    return Qnil;
}

static VALUE
set_http_version(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_iv_set(self, "@http_version", v);
    curl_easy_setopt(s->curl, CURLOPT_HTTP_VERSION, FIX2LONG(v));
    return Qnil;
}

static VALUE
set_ftp_use_epsv(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@ftp_use_epsv", v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, CURLOPT_FTP_USE_EPSV, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, CURLOPT_FTP_USE_EPSV, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}

