#include <ruby.h>
#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>

typedef struct _rbCurl  rbCurl;
typedef struct _pFILE   pFILE;

struct _pFILE {
    char   *pt;
    size_t len;
};

struct _rbCurl {
    CURL   *curl;
    /*VALUE  prms;*/    /* ref to Hash */

    VALUE  body_str;   /* strings */
    VALUE  header_str; /* strings */
    VALUE  input_str;  /* strings */
    /*VALUE  progfunc;*/  /* ref to Block */
    /*VALUE  passfunc;*/  /* ref to Block */

    /*pFILE  fin;*/                /* pseudo FILE */
    /*FILE   *file_file;*/         /* when use FILE */
    /*FILE   *file_writeheader;*/  /* when use FILE */
    /*FILE   *file_infile;*/       /* when use FILE */
    /*FILE   *file_stderr;*/       /* when use FILE */

    struct curl_slist  *list_httpheader;
    struct curl_slist  *list_quote;
    struct curl_slist  *list_postquote;
    struct HttpPost    *list_httppost;
};

#define GetRbCurl(self, pt)  Data_Get_Struct(self, rbCurl, pt)

static VALUE  cCurl;

/***************************************************************/

#include "auto_funcs.inc"

/***************************************************************/

static VALUE
set_httppost(VALUE self, VALUE v)
{
    rbCurl *s;
    int    i;
    struct HttpPost *last = NULL;

    rb_warn("httppost is using curl_formparse(). obsolete");
 
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@httppost", v);

    if (s->list_httppost) { curl_formfree(s->list_httppost); }
    for (i = 0; i < RARRAY(v)->len; i++) {
        curl_formparse(rb_str2cstr(RARRAY(v)->ptr[i], 0), &s->list_httppost, &last);
    }

    if (s->list_httppost)   { curl_easy_setopt(s->curl, CURLOPT_HTTPPOST, s->list_httppost); }
    return Qnil;
}

/************************************************************************/

static int
progress_func(VALUE proc, size_t dltotal, size_t dlnow, size_t ultotal, size_t ulnow)
{
    rb_funcall(proc, rb_intern("call"), 4, INT2FIX(dltotal),INT2FIX(dlnow),INT2FIX(ultotal),INT2FIX(ulnow));
    return 0;
}

static VALUE
set_progressfunction(VALUE self, VALUE v)
{
    rbCurl *s;
    int i;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@progressfunction", v);

    if (v != Qnil) {
	curl_easy_setopt(s->curl, CURLOPT_PROGRESSFUNCTION, progress_func);
	curl_easy_setopt(s->curl, CURLOPT_PROGRESSDATA, v);
	curl_easy_setopt(s->curl, CURLOPT_NOPROGRESS, FALSE);
    } else {
	curl_easy_setopt(s->curl, CURLOPT_PROGRESSFUNCTION, NULL);
	curl_easy_setopt(s->curl, CURLOPT_PROGRESSDATA, NULL);
	curl_easy_setopt(s->curl, CURLOPT_NOPROGRESS, TRUE);
    }

    return Qnil;
}

/************************************************************************/

static int
passwd_func(VALUE proc, char *prompt, char *buffer, int buflen)
{
    VALUE passwd;
    char  *str;
    int   len;
    passwd = rb_funcall(proc, rb_intern("call"), 1, rb_str_new2(prompt));
    str = rb_str2cstr(passwd, &len);
    if (len >= buflen) { return -1; }
    memcpy(buffer, str, len);
    buffer[len] = '\0';
    return 0;
}

static VALUE
set_passwdfunction(VALUE self, VALUE v)
{
    rbCurl *s;
    int i;
    Data_Get_Struct(self, rbCurl, s);
    rb_iv_set(self, "@progressfunction", v);
    curl_easy_setopt(s->curl, CURLOPT_PASSWDFUNCTION, passwd_func);
    curl_easy_setopt(s->curl, CURLOPT_PASSWDDATA, v);
    return Qnil;
}

/************************************************************************/
/************************************************************************/

static void
mark_rbCurl(rbCurl *s)
{
    /*rb_gc_mark(s->prms);*/
    rb_gc_mark(s->body_str);
    rb_gc_mark(s->header_str);
    rb_gc_mark(s->input_str);
    /*rb_gc_mark(s->progfunc);*/
    /*rb_gc_mark(s->passfunc);*/
}

static void
free_rbCurl(rbCurl *s)
{
    curl_slist_free_all(s->list_httpheader);
    curl_slist_free_all(s->list_quote);
    curl_slist_free_all(s->list_postquote);
    if (s->list_httppost)   { curl_formfree(s->list_httppost); }

    curl_easy_cleanup(s->curl);
    free(s);
}

/************************************************************************/

static size_t
write_func(void *src, size_t size, size_t nmemb, VALUE *rbstr)
{
    long len = size * nmemb;
    rb_str_cat(*rbstr, src, len);
    return len;
}

static size_t 
read_func(void *dst, size_t size, size_t nmemb, pFILE *f)
{
    size_t  max_len = size * nmemb;
    size_t  len;

    if (!f->len) { return 0; }
    len = f->len;
    if (f->len > max_len) { len = max_len; }
    memcpy(dst, f->pt, len);
    f->len -= len;
    f->pt += len;
    return len;
}


static VALUE
new(VALUE klass)
{
    rbCurl  *s;
    VALUE   obj;

    obj = Data_Make_Struct(klass, rbCurl, mark_rbCurl, free_rbCurl, s);
    s->curl = curl_easy_init();
    /*s->prms = rb_hash_new();*/

    s->body_str = rb_str_new(0, 0);
    s->header_str = rb_str_new(0, 0);
    s->input_str = rb_str_new(0, 0);
    /*s->progfunc = Qnil;*/
    /*s->passfunc = Qnil;*/

    /*s->file_file = NULL;*/
    /*s->file_writeheader = NULL;*/
    /*s->file_infile = NULL;*/
    /*s->file_stderr = NULL;*/

    s->list_httpheader = NULL;
    s->list_quote = NULL;
    s->list_postquote = NULL;
    s->list_httppost = NULL;

    /*curl_easy_setopt(s->curl, CURLOPT_MUTE, TRUE);*/
    return obj;
}

/***************************************************************/

static VALUE
perform(VALUE self)
{
    rbCurl    *s;
    CURLcode  result;
    char      errbuff[CURL_ERROR_SIZE] = "";
    VALUE     v;
    int       len;
    FILE      *f;
    pFILE     fin;                /* pseudo FILE */
    FILE      *file_file = NULL;
    FILE      *file_writeheader = NULL;
    FILE      *file_infile = NULL;
    FILE      *file_stderr = NULL;

    GetRbCurl(self, s);

    v = rb_iv_get(self, "@file");
    if (v != Qnil) {
	f = fopen(rb_str2cstr(v, 0), "w");
	if (!f) { rb_raise(rb_eRuntimeError, "can't open file"); }
	curl_easy_setopt(s->curl, CURLOPT_FILE, f);
	file_file = f;  /* keep for close */
    }
    v = rb_iv_get(self, "@infile");
    if (v != Qnil) {
	f = fopen(rb_str2cstr(v, 0), "r");
	if (!f) { rb_raise(rb_eRuntimeError, "can't open file"); }
	curl_easy_setopt(s->curl, CURLOPT_INFILE, f);
	file_infile = f;  /* keep for close */
    }
    v =  rb_iv_get(self, "@writeheader");
    if (v != Qnil) {
	f = fopen(rb_str2cstr(v, 0), "w");
	if (!f) { rb_raise(rb_eRuntimeError, "can't open file"); }
	curl_easy_setopt(s->curl, CURLOPT_WRITEHEADER, f);
	file_writeheader = f;  /* keep for close */
    }
    v = rb_iv_get(self, "@stderr");
    if (v != Qnil) {
	f = fopen(rb_str2cstr(v, 0), "w");
	if (!f) { rb_raise(rb_eRuntimeError, "can't open file"); }
	curl_easy_setopt(s->curl, CURLOPT_STDERR, f);
	file_stderr = f;  /* keep for close */
    }



    /* No file for body and header */
    if ((!file_file) && (!file_writeheader)) {
	/* use Ruby String  */
	s->body_str = rb_str_new(0, 0);     /* new */
	s->header_str = rb_str_new(0, 0);   /* new */
	curl_easy_setopt(s->curl, CURLOPT_FILE, &(s->body_str));
	curl_easy_setopt(s->curl, CURLOPT_WRITEFUNCTION, write_func);
	curl_easy_setopt(s->curl, CURLOPT_WRITEHEADER, &(s->header_str));
	curl_easy_setopt(s->curl, CURLOPT_HEADER, FALSE);
    }

    /* No file for input */
    if (!file_infile) {
	/* use Ruby String  */
	fin.pt = rb_str2cstr(s->input_str, &len);
	if (len) {  /* input exist */
	    fin.len = len;
	    curl_easy_setopt(s->curl, CURLOPT_INFILE, &fin);
	    curl_easy_setopt(s->curl, CURLOPT_READFUNCTION, read_func);
	    curl_easy_setopt(s->curl, CURLOPT_INFILESIZE, len);
	    curl_easy_setopt(s->curl, CURLOPT_UPLOAD, TRUE);
	}
    }

    if (file_stderr) {
	curl_easy_setopt(s->curl, CURLOPT_STDERR, file_stderr);
    }


    curl_easy_setopt(s->curl, CURLOPT_ERRORBUFFER, errbuff);
    result = curl_easy_perform(s->curl);

    if (file_file) { fclose(file_file); }
    if (file_writeheader) { fclose(file_writeheader); }
    if (file_infile) { fclose(file_infile); }
    if (file_stderr) { fclose(file_stderr); }

    if (result != CURLE_OK) {
	rb_raise(rb_eRuntimeError, errbuff);
    }

    return self;
}

/************************************************************************/

static VALUE
get_body_str(VALUE self)
{
    rbCurl    *c;
    VALUE     v;
    GetRbCurl(self, c);
    return c->body_str;
}

static VALUE
get_header_str(VALUE self)
{
    rbCurl    *c;
    VALUE     v;
    GetRbCurl(self, c);
    return c->header_str;
}

static VALUE
set_input_str(VALUE self, VALUE v)
{
    rbCurl *s;
    GetRbCurl(self, s);
    s->input_str = v;
    return Qnil;
}

static VALUE
get_input_str(VALUE self)
{
    rbCurl    *s;
    GetRbCurl(self, s);
    return s->input_str;
}


/***************************************************************/
static VALUE
test(VALUE self, VALUE v)
{
    rbCurl *s;
    GetRbCurl(self, s);

    rb_yield(rb_str_new2("hekomaka"));
    return Qnil;
}

void
Init_curl(void)
{
   cCurl = rb_define_class("Curl", rb_cObject);

   rb_define_singleton_method(cCurl, "new", new, 0);
   /*rb_define_method(cCurl, "prms",  get_prms, 0);*/

   rb_define_method(cCurl, "perform", perform, 0);
   rb_define_method(cCurl, "body_str", get_body_str, 0);
   rb_define_method(cCurl, "header_str", get_header_str, 0);
   rb_define_method(cCurl, "input_str=", set_input_str, 1);
   rb_define_method(cCurl, "input_str", get_input_str, 0);

   rb_define_attr(cCurl, "file", 1,1);
   rb_define_attr(cCurl, "infile", 1,1);
   rb_define_attr(cCurl, "writeheader", 1,1);
   rb_define_attr(cCurl, "stderr", 1,1);

   rb_define_method(cCurl, "httppost=", set_httppost, 1);
   rb_define_attr(cCurl, "httppost", 1,0);

   rb_define_method(cCurl, "progressfunction=", set_progressfunction, 1);
   rb_define_attr(cCurl, "progressfunction", 1,0);
   rb_define_method(cCurl, "passwdfunction=", set_passwdfunction, 1);
   rb_define_attr(cCurl, "passwdfunction", 1,0);

   rb_define_method(cCurl, "test", test, 0);

/***************************************************************/
#include "auto_defs.inc"
/***************************************************************/

   rb_define_const(cCurl, "TIMECOND_NONE", INT2FIX(TIMECOND_NONE));
   rb_define_const(cCurl, "TIMECOND_IFMODSINCE", INT2FIX(TIMECOND_IFMODSINCE));
   rb_define_const(cCurl, "TIMECOND_IFUNMODSINCE", INT2FIX(TIMECOND_IFUNMODSINCE));
   rb_define_const(cCurl, "TIMECOND_LASTMOD", INT2FIX(TIMECOND_LASTMOD));

   rb_define_const(cCurl, "HTTP_VERSION_NONE", INT2FIX(CURL_HTTP_VERSION_NONE));
   rb_define_const(cCurl, "HTTP_VERSION_1_0", INT2FIX(CURL_HTTP_VERSION_1_0));
   rb_define_const(cCurl, "HTTP_VERSION_1_1", INT2FIX(CURL_HTTP_VERSION_1_1));

   rb_define_const(cCurl, "SSLVERSION_DEFAULT", INT2FIX(CURL_SSLVERSION_DEFAULT));
   rb_define_const(cCurl, "SSLVERSION_TLSv1", INT2FIX(CURL_SSLVERSION_TLSv1));
   rb_define_const(cCurl, "SSLVERSION_SSLv2", INT2FIX(CURL_SSLVERSION_SSLv2));
   rb_define_const(cCurl, "SSLVERSION_SSLv3", INT2FIX(CURL_SSLVERSION_SSLv3));
}
